@@ -60,7 +60,7 @@ diskio::gen_txt_header(const Mat<eT>& x)
     {
     return std::string("ARMA_MAT_TXT_IS004");
     }
-#if defined(ARMA_64BIT_WORD)
+#if defined(ARMA_USE_U64S64)
   else
   if(is_u64<eT>::value == true)
     {
@@ -71,6 +71,28 @@ diskio::gen_txt_header(const Mat<eT>& x)
     {
     return std::string("ARMA_MAT_TXT_IS008");
     }
+#endif
+#if defined(ARMA_ALLOW_LONG)
+  else
+  if(is_ulng_t_32<eT>::value == true)
+    {
+    return std::string("ARMA_MAT_TXT_IU004");
+    }
+  else
+  if(is_slng_t_32<eT>::value == true)
+    {
+    return std::string("ARMA_MAT_TXT_IS004");
+    }
+  else
+  if(is_ulng_t_64<eT>::value == true)
+    {
+    return std::string("ARMA_MAT_TXT_IU008");
+    }
+  else
+  if(is_slng_t_64<eT>::value == true)
+    {
+    return std::string("ARMA_MAT_TXT_IS008");
+    }
 #endif
   else
   if(is_float<eT>::value == true)
@@ -144,7 +166,7 @@ diskio::gen_bin_header(const Mat<eT>& x)
     {
     return std::string("ARMA_MAT_BIN_IS004");
     }
-#if defined(ARMA_64BIT_WORD)
+#if defined(ARMA_USE_U64S64)
   else
   if(is_u64<eT>::value == true)
     {
@@ -155,6 +177,28 @@ diskio::gen_bin_header(const Mat<eT>& x)
     {
     return std::string("ARMA_MAT_BIN_IS008");
     }
+#endif
+#if defined(ARMA_ALLOW_LONG)
+  else
+  if(is_ulng_t_32<eT>::value == true)
+    {
+    return std::string("ARMA_MAT_BIN_IU004");
+    }
+  else
+  if(is_slng_t_32<eT>::value == true)
+    {
+    return std::string("ARMA_MAT_BIN_IS004");
+    }
+  else
+  if(is_ulng_t_64<eT>::value == true)
+    {
+    return std::string("ARMA_MAT_BIN_IU008");
+    }
+  else
+  if(is_slng_t_64<eT>::value == true)
+    {
+    return std::string("ARMA_MAT_BIN_IS008");
+    }
 #endif
   else
   if(is_float<eT>::value == true)
@@ -228,7 +272,7 @@ diskio::gen_bin_header(const SpMat<eT>& x)
     {
     return std::string("ARMA_SPM_BIN_IS004");
     }
-#if defined(ARMA_64BIT_WORD)
+#if defined(ARMA_USE_U64S64)
   else
   if(is_u64<eT>::value == true)
     {
@@ -239,6 +283,28 @@ diskio::gen_bin_header(const SpMat<eT>& x)
     {
     return std::string("ARMA_SPM_BIN_IS008");
     }
+#endif
+#if defined(ARMA_ALLOW_LONG)
+  else
+  if(is_ulng_t_32<eT>::value == true)
+    {
+    return std::string("ARMA_SPM_BIN_IU004");
+    }
+  else
+  if(is_slng_t_32<eT>::value == true)
+    {
+    return std::string("ARMA_SPM_BIN_IS004");
+    }
+  else
+  if(is_ulng_t_64<eT>::value == true)
+    {
+    return std::string("ARMA_SPM_BIN_IU008");
+    }
+  else
+  if(is_slng_t_64<eT>::value == true)
+    {
+    return std::string("ARMA_SPM_BIN_IS008");
+    }
 #endif
   else
   if(is_float<eT>::value == true)
@@ -311,7 +377,7 @@ diskio::gen_txt_header(const Cube<eT>& x)
     {
     return std::string("ARMA_CUB_TXT_IS004");
     }
-#if defined(ARMA_64BIT_WORD)
+#if defined(ARMA_USE_U64S64)
   else
   if(is_u64<eT>::value == true)
     {
@@ -322,6 +388,28 @@ diskio::gen_txt_header(const Cube<eT>& x)
     {
     return std::string("ARMA_CUB_TXT_IS008");
     }
+#endif
+#if defined(ARMA_ALLOW_LONG)
+  else
+  if(is_ulng_t_32<eT>::value == true)
+    {
+    return std::string("ARMA_CUB_TXT_IU004");
+    }
+  else
+  if(is_slng_t_32<eT>::value == true)
+    {
+    return std::string("ARMA_CUB_TXT_IS004");
+    }
+  else
+  if(is_ulng_t_64<eT>::value == true)
+    {
+    return std::string("ARMA_CUB_TXT_IU008");
+    }
+  else
+  if(is_slng_t_64<eT>::value == true)
+    {
+    return std::string("ARMA_CUB_TXT_IS008");
+    }
 #endif
   else
   if(is_float<eT>::value == true)
@@ -395,7 +483,7 @@ diskio::gen_bin_header(const Cube<eT>& x)
     {
     return std::string("ARMA_CUB_BIN_IS004");
     }
-#if defined(ARMA_64BIT_WORD)
+#if defined(ARMA_USE_U64S64)
   else
   if(is_u64<eT>::value == true)
     {
@@ -406,6 +494,28 @@ diskio::gen_bin_header(const Cube<eT>& x)
     {
     return std::string("ARMA_CUB_BIN_IS008");
     }
+#endif
+#if defined(ARMA_ALLOW_LONG)
+  else
+  if(is_ulng_t_32<eT>::value == true)
+    {
+    return std::string("ARMA_CUB_BIN_IU004");
+    }
+  else
+  if(is_slng_t_32<eT>::value == true)
+    {
+    return std::string("ARMA_CUB_BIN_IS004");
+    }
+  else
+  if(is_ulng_t_64<eT>::value == true)
+    {
+    return std::string("ARMA_CUB_BIN_IU008");
+    }
+  else
+  if(is_slng_t_64<eT>::value == true)
+    {
+    return std::string("ARMA_CUB_BIN_IS008");
+    }
 #endif
   else
   if(is_float<eT>::value == true)
