@@ -1,5 +1,5 @@
-// Copyright (C) 2008-2010 NICTA (www.nicta.com.au)
-// Copyright (C) 2008-2010 Conrad Sanderson
+// Copyright (C) 2008-2012 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2012 Conrad Sanderson
 // 
 // This file is part of the Armadillo C++ library.
 // It is provided without any warranty of fitness
@@ -45,7 +45,6 @@ out_eT
 conv_to<out_eT>::from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   arma_type_check(( is_supported_elem_type<out_eT>::value == false ));
@@ -67,7 +66,6 @@ out_eT
 conv_to<out_eT>::from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   arma_type_check(( is_supported_elem_type<out_eT>::value == false ));
@@ -93,7 +91,6 @@ out_eT
 conv_to<out_eT>::from(const BaseCube<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   arma_type_check(( is_supported_elem_type<out_eT>::value == false ));
@@ -115,7 +112,6 @@ out_eT
 conv_to<out_eT>::from(const BaseCube<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   arma_type_check(( is_supported_elem_type<out_eT>::value == false ));
@@ -164,7 +160,6 @@ Mat<out_eT>
 conv_to< Mat<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   const unwrap<T1>      tmp(in.get_ref());
@@ -186,7 +181,6 @@ Mat<out_eT>
 conv_to< Mat<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   const unwrap<T1>      tmp(in.get_ref());
@@ -208,23 +202,21 @@ Mat<out_eT>
 conv_to< Mat<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
-  Mat<out_eT> out(in.size(), 1);
+  const uword N = uword( in.size() );
   
-  typename std::vector<in_eT>::const_iterator in_begin = in.begin();
-  typename std::vector<in_eT>::const_iterator in_end   = in.end();
+  Mat<out_eT> out(N, 1);
   
-  typename Mat<out_eT>::iterator out_begin = out.begin();
-  typename Mat<out_eT>::iterator out_end   = out.end();
-  
-  typename std::vector<in_eT>::const_iterator in_it;
-  typename Mat<out_eT>::iterator              out_it;
-  
-  for(in_it = in_begin, out_it = out_begin; (in_it != in_end) && (out_it != out_end); ++in_it, ++out_it)
+  if(N > 0)
     {
-    (*out_it) = out_eT(*in_it);
+          out_eT* out_mem = out.memptr();
+    const  in_eT*  in_mem = &(in[0]);
+    
+    for(uword i=0; i<N; ++i)
+      {
+      out_mem[i] = out_eT( in_mem[i] );
+      }
     }
   
   return out;
@@ -239,7 +231,6 @@ Mat<out_eT>
 conv_to< Mat<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   Mat<out_eT> out(in.size(), 1);
@@ -296,7 +287,6 @@ Row<out_eT>
 conv_to< Row<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   const unwrap<T1>      tmp(in.get_ref());
@@ -320,7 +310,6 @@ Row<out_eT>
 conv_to< Row<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   const unwrap<T1>      tmp(in.get_ref());
@@ -344,23 +333,21 @@ Row<out_eT>
 conv_to< Row<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
-  Row<out_eT> out( in.size() );
+  const uword N = uword( in.size() );
   
-  typename std::vector<in_eT>::const_iterator in_begin = in.begin();
-  typename std::vector<in_eT>::const_iterator in_end   = in.end();
+  Row<out_eT> out(N);
   
-  typename Row<out_eT>::iterator out_begin = out.begin();
-  typename Row<out_eT>::iterator out_end   = out.end();
-  
-  typename std::vector<in_eT>::const_iterator in_it;
-  typename Row<out_eT>::iterator              out_it;
-  
-  for(in_it = in_begin, out_it = out_begin; (in_it != in_end) && (out_it != out_end); ++in_it, ++out_it)
+  if(N > 0)
     {
-    (*out_it) = out_eT(*in_it);
+          out_eT* out_mem = out.memptr();
+    const  in_eT*  in_mem = &(in[0]);
+    
+    for(uword i=0; i<N; ++i)
+      {
+      out_mem[i] = out_eT( in_mem[i] );
+      }
     }
   
   return out;
@@ -375,7 +362,6 @@ Row<out_eT>
 conv_to< Row<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   Row<out_eT> out( in.size() );
@@ -432,7 +418,6 @@ Col<out_eT>
 conv_to< Col<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   const unwrap<T1>      tmp(in.get_ref());
@@ -456,7 +441,6 @@ Col<out_eT>
 conv_to< Col<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   const unwrap<T1>      tmp(in.get_ref());
@@ -480,23 +464,21 @@ Col<out_eT>
 conv_to< Col<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
-  Col<out_eT> out( in.size() );
+  const uword N = uword( in.size() );
   
-  typename std::vector<in_eT>::const_iterator in_begin = in.begin();
-  typename std::vector<in_eT>::const_iterator in_end   = in.end();
+  Col<out_eT> out(N);
   
-  typename Col<out_eT>::iterator out_begin = out.begin();
-  typename Col<out_eT>::iterator out_end   = out.end();
-  
-  typename std::vector<in_eT>::const_iterator in_it;
-  typename Col<out_eT>::iterator              out_it;
-  
-  for(in_it = in_begin, out_it = out_begin; (in_it != in_end) && (out_it != out_end); ++in_it, ++out_it)
+  if(N > 0)
     {
-    (*out_it) = out_eT(*in_it);
+          out_eT* out_mem = out.memptr();
+    const  in_eT*  in_mem = &(in[0]);
+    
+    for(uword i=0; i<N; ++i)
+      {
+      out_mem[i] = out_eT( in_mem[i] );
+      }
     }
   
   return out;
@@ -511,7 +493,6 @@ Col<out_eT>
 conv_to< Col<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   Col<out_eT> out( in.size() );
@@ -560,7 +541,6 @@ Cube<out_eT>
 conv_to< Cube<out_eT> >::from(const BaseCube<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   const unwrap_cube<T1>  tmp( in.get_ref() );
@@ -582,7 +562,6 @@ Cube<out_eT>
 conv_to< Cube<out_eT> >::from(const BaseCube<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   const unwrap_cube<T1>  tmp( in.get_ref() );
@@ -619,7 +598,6 @@ std::vector<out_eT>
 conv_to< std::vector<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   const unwrap<T1>      tmp(in.get_ref());
@@ -627,20 +605,19 @@ conv_to< std::vector<out_eT> >::from(const Base<in_eT, T1>& in, const typename a
   
   arma_debug_check( (X.is_vec() == false), "conv_to(): given object can't be interpreted as a vector" );
   
-  std::vector<out_eT> out(X.n_elem);
+  const uword N = X.n_elem;
   
-  typename Mat<in_eT>::const_iterator X_begin = X.begin();
-  typename Mat<in_eT>::const_iterator X_end   = X.end();
-
-  typename std::vector<out_eT>::iterator out_begin = out.begin();
-  typename std::vector<out_eT>::iterator out_end   = out.end();
+  std::vector<out_eT> out(N);
   
-  typename Mat<in_eT>::const_iterator    X_it;
-  typename std::vector<out_eT>::iterator out_it;
-  
-  for(X_it = X_begin, out_it = out_begin; (X_it != X_end) && (out_it != out_end); ++X_it, ++out_it)
+  if(N > 0)
     {
-    (*out_it) = out_eT(*X_it);
+          out_eT* out_mem = &(out[0]);
+    const  in_eT*   X_mem = X.memptr();
+    
+    for(uword i=0; i<N; ++i)
+      {
+      out_mem[i] = out_eT( X_mem[i] );
+      }
     }
   
   return out;
@@ -655,7 +632,6 @@ std::vector<out_eT>
 conv_to< std::vector<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
-  
   arma_ignore(junk);
   
   const unwrap<T1>      tmp(in.get_ref());
