@@ -1,5 +1,5 @@
-// Copyright (C) 2008-2011 NICTA (www.nicta.com.au)
-// Copyright (C) 2008-2011 Conrad Sanderson
+// Copyright (C) 2008-2013 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2013 Conrad Sanderson
 // 
 // This file is part of the Armadillo C++ library.
 // It is provided without any warranty of fitness
@@ -15,10 +15,12 @@
 //! @{
 
 
-#if UCHAR_MAX >= 0xff
-  //! unsigned 8 bit type
-  typedef unsigned char u8;
-  typedef          char s8;
+#if   UCHAR_MAX >= 0xff
+  typedef unsigned char    u8;
+  typedef          char    s8;
+#elif defined(UINT8_MAX)
+  typedef          uint8_t u8;
+  typedef           int8_t s8;
 #else
   #error "don't know how to typedef 'u8' on this system"
 #endif
@@ -29,56 +31,52 @@
 // http://en.wikipedia.org/wiki/C_variable_types_and_declarations
 
 
-#if USHRT_MAX >= 0xffff
-  //! unsigned 16 bit type  
-  typedef unsigned short u16;
-  typedef          short s16;
+#if   USHRT_MAX >= 0xffff
+  typedef unsigned short    u16;
+  typedef          short    s16;
+#elif defined(UINT16_MAX)
+  typedef          uint16_t u16;
+  typedef           int16_t s16;
 #else
   #error "don't know how to typedef 'u16' on this system"
 #endif
 
 
 #if   UINT_MAX  >= 0xffffffff
-  typedef unsigned int  u32;
-  typedef          int  s32;
-#elif ULONG_MAX >= 0xffffffff
-  typedef unsigned long u32;
-  typedef          long s32;
+  typedef unsigned int      u32;
+  typedef          int      s32;
+#elif defined(UINT32_MAX)
+  typedef          uint32_t u32;
+  typedef           int32_t s32;
 #else
   #error "don't know how to typedef 'u32' on this system"
 #endif
 
 
-#if defined(ARMA_64BIT_WORD)
-  #if    ULONG_MAX >= 0xffffffffffffffff
+#if defined(ARMA_USE_U64S64)
+  #if   ULLONG_MAX >= 0xffffffffffffffff
+    typedef unsigned long long u64;
+    typedef          long long s64;
+  #elif ULONG_MAX  >= 0xffffffffffffffff
     typedef unsigned long      u64;
     typedef          long      s64;
+    #define ARMA_U64_IS_LONG
+  #elif defined(UINT64_MAX)
+    typedef          uint64_t  u64;
+    typedef           int64_t  s64;
   #else
-    #if ULLONG_MAX >= 0xffffffffffffffff
-      typedef unsigned long long u64;
-      typedef          long long s64;
-    #else
-      #error "don't know how to typedef 'u64' on this system"
-    #endif
+      #error "don't know how to typedef 'u64' on this system; please disable ARMA_64BIT_WORD and/or ARMA_USE_U64S64"
   #endif
 #endif
 
 
+#if !defined(ARMA_USE_U64S64) || (defined(ARMA_USE_U64S64) && !defined(ARMA_U64_IS_LONG))
+  #define ARMA_ALLOW_LONG
+#endif
 
-// // only supported by C++11, via #include <cstdint>, or by C99, via #include <stdint.h>
-// 
-// typedef  uint8_t u8;
-// typedef   int8_t s8;
-// 
-// typedef uint16_t u16;
-// typedef  int16_t s16;
-// 
-// typedef uint32_t u32;
-// typedef  int32_t s32;
-// 
-// typedef uint64_t u64;
-// typedef  int64_t s64;
 
+typedef unsigned long ulng_t;
+typedef          long slng_t;
 
 
 #if !defined(ARMA_64BIT_WORD)
@@ -124,6 +122,20 @@ typedef Col <s32> s32_colvec;
 typedef Row <s32> s32_rowvec;
 typedef Cube<s32> s32_cube;
 
+#if defined(ARMA_USE_U64S64)
+  typedef Mat <u64> u64_mat;
+  typedef Col <u64> u64_vec;
+  typedef Col <u64> u64_colvec;
+  typedef Row <u64> u64_rowvec;
+  typedef Cube<u64> u64_cube;
+
+  typedef Mat <s64> s64_mat;
+  typedef Col <s64> s64_vec;
+  typedef Col <s64> s64_colvec;
+  typedef Row <s64> s64_rowvec;
+  typedef Cube<s64> s64_cube;
+#endif
+
 typedef Mat <uword> umat;
 typedef Col <uword> uvec;
 typedef Col <uword> ucolvec;
@@ -212,9 +224,9 @@ namespace junk
     arma_static_check( (sizeof(u32) != 4), ERROR___TYPE_U32_HAS_UNSUPPORTED_SIZE );
     arma_static_check( (sizeof(s32) != 4), ERROR___TYPE_S32_HAS_UNSUPPORTED_SIZE );
     
-    #if defined(ARMA_64BIT_WORD)
-    arma_static_check( (sizeof(u64) != 8), ERROR___TYPE_U64_HAS_UNSUPPORTED_SIZE );
-    arma_static_check( (sizeof(s64) != 8), ERROR___TYPE_S64_HAS_UNSUPPORTED_SIZE );
+    #if defined(ARMA_USE_U64S64)
+      arma_static_check( (sizeof(u64) != 8), ERROR___TYPE_U64_HAS_UNSUPPORTED_SIZE );
+      arma_static_check( (sizeof(s64) != 8), ERROR___TYPE_S64_HAS_UNSUPPORTED_SIZE );
     #endif
     
     arma_static_check( (sizeof(float)  != 4), ERROR___TYPE_FLOAT_HAS_UNSUPPORTED_SIZE );
